unit _ipAUTOINC;

interface

uses
{$ifdef WIN32}
  ComCtrls, CommCtrl,
{$endif}
  Windows, WinTypes, Forms, Dialogs, WinProcs, Messages, SysUtils, Classes,
  Graphics, Controls, StdCtrls, DBCtrls, Grids, DBGrids, Db, DBTables, _IP;

type
   TUpdateActions=(uaAfterInsert,uaBeforePost);

  Tip_AUTOINC = class(TComponent)
  private
     FBeforePostEvent:TDataSetNotifyEvent;
     FAfterInsertEvent:TDataSetNotifyEvent;
     FPrefix:String;
     FFirstID: Integer;
     FDataset:TDataset;
     FEnabled: Boolean;
     FUpdateAction:TUpdateActions;
     function GetDataset: TDataset;
     procedure SetDataset(Value: TDataset);
     procedure AfterInsert(DataSet: TDataSet);
     procedure BeforePost(DataSet: TDataSet);
     function FirstField:TField;
    { Private declarations }
  protected
    { Protected declarations }
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Loaded; override;
   public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function LastID:integer;
    procedure WriteID(ID:integer);
  published
    { Published declarations }
    //property DataField: string read GetDataField write SetDataField;
    property DataSet: TDataset read GetDataset write SetDataset;
    property Prefix: String read FPrefix write FPrefix;
    property FirstID: Integer read FFirstID write FFirstID;
    property Enabled: Boolean read FEnabled write FEnabled;
    property UpdateAction: TUpdateActions read FUpdateAction write FUpdateAction;
  end;

procedure Register;

implementation

constructor Tip_AUTOINC.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FUpdateAction:= uaAfterInsert;
  Enabled:=True;
  FirstID:=1;
end;

destructor Tip_AUTOINC.Destroy;
begin
  inherited Destroy;
end;

procedure Tip_AUTOINC.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState)then
  begin
      FAfterInsertEvent:=(Dataset as TTable).AfterInsert;
      (Dataset as TTable).AfterInsert:=AfterInsert;

      FBeforePostEvent:=(Dataset as TTable).BeforePost;
      (Dataset as TTable).BeforePost:=BeforePost;
  end;
end;

procedure Tip_AUTOINC.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove)  and (FDataset <> nil) and
    (AComponent = DataSet) then Dataset := nil;
end;

function Tip_AUTOINC.GetDataset: TDataset;
begin
  Result := FDataset;
end;

procedure Tip_AUTOINC.SetDataset(Value: TDataset);
begin
  FDataset := Value;
//   if Value <> nil then Value.FreeNotification(Self);
end;


procedure Tip_AUTOINC.AfterInsert(DataSet: TDataSet);
begin
   if Enabled and FirstField.Required and (FUpdateAction=uaBeforePost)then
   begin
      ShowMessage('First field must not be Required');
   end;

   if Enabled and (FUpdateAction=uaAfterInsert) then
   begin
       if LastID=0 then
         WriteID(FirstID)
      else
         WriteID(LastID+1)
   end;
   if assigned(FAfterInsertEvent) then
      FAfterInsertEvent(DataSet);
end;

procedure Tip_AUTOINC.BeforePost(DataSet: TDataSet);
begin
   if Enabled and (FUpdateAction=uaBeforePost) then
   begin
      if LastID=0 then
         WriteID(FirstID)
      else
         WriteID(LastID+1)
   end;
   if assigned(FBeforePostEvent) then
      FBeforePostEvent(DataSet);
end;

function Tip_AUTOINC.LastID:integer;
var
   Table1:TTable;
   AString:String;
begin
   try
      Screen.Cursor:=crHourglass;
      Table1:=TTable.Create(self);
      Table1.DatabaseName:=(Dataset as TTable).DatabaseName;
      Table1.TableName:=(Dataset as TTable).TableName;
      Table1.Open;
      Table1.Last;
      if (Table1.BOF) and (Table1.EOF) then
         Result:=0
      else if Table1.Fields[0].DataType=ftInteger	then
         Result:=Table1.Fields[0].AsInteger
      else if Table1.Fields[0].DataType=ftString then
      begin
         AString:=Table1.Fields[0].AsString;
         if AString='' then
         begin
            Result:=0;
         end
         else
         begin
            AString:=RightS(AString,Length(AString)-Length(Prefix));
            Result:=StrToInt(AString);
         end;
      end
      else
         ShowMessage('First field is not of supported types');
      Table1.Close;
   finally
      Table1.Free;
      Screen.Cursor:=crDefault;
   end;
end;

procedure Tip_AUTOINC.WriteID(ID:integer);
var
   AField:TField;
   a:string;
begin
  if (Dataset as TTable).State = dsInsert then
  begin
      AField:=FirstField;
      if AField.DataType=ftInteger then
         AField.AsInteger:=ID
      else if AField.DataType=ftString then
      begin
         AField.AsString:=Prefix+RightS(MakeS('0',20)+IntToStr(ID),AField.Size-Length(Prefix));
      end;
   end;
end;

function Tip_AUTOINC.FirstField:TField;
var
   FirstFieldName:String;
begin
      FirstFieldName:=(Dataset as TTable).FieldDefs.Items[0].Name;
      Result:=(Dataset as TTable).FieldByName(FirstFieldName);
end;

procedure Register;
begin
  RegisterComponents('_InfoPlus', [Tip_AUTOINC]);
end;

end.
